package Main

import Util;

function startClock
  output Real start;
algorithm
  start := clock();
end startClock;

function stopClock
  input Real start;
  input String message;
  Real stop, time;
  String timeStr;  
algorithm
    stop := clock();
    time := stop -. start;
    timeStr := realString(time);
    print("----> " +& message +& " took: " +& timeStr +& " seconds to execute\n");
end stopClock;

function enableAll
  input list<Integer> integerLst;
  input list<Real>    realLst;
  input list<String>  stringLst;    
  input Integer enableMe;
algorithm 
  _ := matchcontinue(integerLst, realLst, stringLst, enableMe)
		local 
			Real start;
			list<Integer> integerResultLst;
			list<Real>    realResultLst;
			list<String>  stringResultLst;  
       case (integerLst, realLst, stringLst, 0) equation
		print ("================= INTEGERS ============ Util.listMap_impl_0 \n");    
		start = clock(); integerResultLst = Util.listMap_impl_0(integerLst, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_0/list<Integer>");    
		print ("================= REALS =============== Util.listMap_impl_0 \n");            
		start = clock(); realResultLst = Util.listMap_impl_0(realLst, Util.identityReal);
		stopClock(start, "Util.listMap_impl_0/list<Real>");    
		print ("================= STRINGS ============= Util.listMap_impl_0 \n");                        
		start = clock(); stringResultLst = Util.listMap_impl_0(stringLst, Util.identityString);
		stopClock(start, "Util.listMap_impl_0/list<String>");
	   then ();
       case (integerLst, realLst, stringLst, 1) equation
		print ("================= INTEGERS ============ Util.listMap_impl_1 \n");
		start = clock(); integerResultLst = Util.listMap_impl_1(integerLst, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_1/list<Integer>");
		print ("================= REALS =============== Util.listMap_impl_1 \n");        
		start = clock(); realResultLst = Util.listMap_impl_1(realLst, Util.identityReal);
		stopClock(start, "Util.listMap_impl_1/list<Real>");
		print ("================= STRINGS ============= Util.listMap_impl_1 \n");                
		start = clock(); stringResultLst = Util.listMap_impl_1(stringLst, Util.identityString);
		stopClock(start, "Util.listMap_impl_1/list<String>");
	   then ();
       case (integerLst, realLst, stringLst, 2) equation    
		print ("================= INTEGERS ============ Util.listMap_impl_2 \n");
		start = clock(); integerResultLst = Util.listMap_impl_2(integerLst, {}, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_2/list<Integer>");    
		print ("================= REALS =============== Util.listMap_impl_2 \n");
		start = clock(); realResultLst = Util.listMap_impl_2(realLst, {}, Util.identityReal);
		stopClock(start, "Util.listMap_impl_2/list<Real>");    
		print ("================= STRINGS ============= Util.listMap_impl_2 \n");    
		start = clock(); stringResultLst = Util.listMap_impl_2(stringLst, {}, Util.identityString);
		stopClock(start, "Util.listMap_impl_2/list<String>");
       then ();
       case (integerLst, realLst, stringLst, 3) equation
		print ("================= INTEGERS ============ Util.listMap_impl_3 \n");
		start = clock(); integerResultLst = Util.listMap_impl_3(integerLst, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_3/list<Integer>)");    
		print ("================= REALS =============== Util.listMap_impl_3 \n");
		start = clock(); realResultLst = Util.listMap_impl_3(realLst, Util.identityReal);
	    stopClock(start, "Util.listMap_impl_3/list<Real>");
		print ("================= STRINGS ============= Util.listMap_impl_3 \n");    
		start = clock(); stringResultLst = Util.listMap_impl_3(stringLst, Util.identityString);
		stopClock(start, "Util.listMap_impl_3/list<String>");    
	   then ();   
  end matchcontinue;
end enableAll;
  
function timingAll
  input Integer size;
  input Integer enableWhat;
  Real start;
  list<Integer> integerLst, integerResultLst;
  list<Real>    realLst, realResultLst;
  list<String>  stringLst, stringResultLst;  
algorithm
    start := clock(); 
    integerLst := Util.generateIntegerList(size, 0, {}); // generate a list of A LOT of elements
    stopClock(start, "Integer List generation of: " +& intString(size) +& " elements");    
    start := clock(); 
    realLst := Util.generateRealList(size, 0, {}); // generate a list of A LOT of elements
    stopClock(start, "Real List generation of: " +& intString(size) +& " elements");
    start := clock(); 
    stringLst := Util.generateStringList(size, 0, {}); // generate a list of A LOT of elements
    stopClock(start, "String List generation of: " +& intString(size) +& " elements");
    enableAll(integerLst, realLst, stringLst, enableWhat);    
end timingAll;

function enableInteger
  input list<Integer> integerLst;
  input Integer enableMe;
algorithm 
  _ := matchcontinue(integerLst, enableMe)
		local 
			Real start;
			list<Integer> integerResultLst;
       case (integerLst, 0) equation
		print ("================= INTEGERS ============ Util.listMap_impl_0 \n");    
		start = clock(); integerResultLst = Util.listMap_impl_0(integerLst, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_0/list<Integer>");    
	   then ();
       case (integerLst, 1) equation
		print ("================= INTEGERS ============ Util.listMap_impl_1 \n");
		start = clock(); integerResultLst = Util.listMap_impl_1(integerLst, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_1/list<Integer>");
	   then ();
       case (integerLst, 2) equation    
		print ("================= INTEGERS ============ Util.listMap_impl_2 \n");
		start = clock(); integerResultLst = Util.listMap_impl_2(integerLst, {}, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_2/list<Integer>");    
       then ();
       case (integerLst, 3) equation
		print ("================= INTEGERS ============ Util.listMap_impl_3 \n");
		start = clock(); integerResultLst = Util.listMap_impl_3(integerLst, Util.identityInteger);
		stopClock(start, "Util.listMap_impl_3/list<Integer>)");    
	   then ();   
  end matchcontinue;
end enableInteger;

function timingInteger
  input Integer size;
  input Integer enableWhat;
  Real start;
  list<Integer> integerLst, integerResultLst;
  list<Real>    realLst, realResultLst;
  list<String>  stringLst, stringResultLst;  
algorithm
    start := clock(); 
    integerLst := Util.generateIntegerList(size, 0, {}); // generate a list of A LOT of elements
    stopClock(start, "Integer List generation of: " +& intString(size) +& " elements");    
    enableInteger(integerLst, enableWhat);    
end timingInteger;
    
public function main
  input list<String> inArgs;
algorithm
  _ := matchcontinue(inArgs)
       local String arg, implementationNumber;
       case arg::implementationNumber::{}
        equation
         timingAll(stringInt(arg), stringInt(implementationNumber)); // time the implementation for all String, Integer and Real
        then ();
       case arg::implementationNumber::_
        equation
         timingInteger(stringInt(arg), stringInt(implementationNumber)); // time the implementation for Integer only
        then ();
       case _ equation
        print("Usage: listMapBenchmark NoOfListElements using_implementation:0|1|2|3 integerListOnly\n");
        print("  Examples: \n\t./listMapBenchmark 50000 1");
        print("\n\t./listMapBenchmark -log 50000 2");
        print("\n\t./listMapBenchmark -log -young-size=10M 4000000 3");
        print("\n\t See also the test-listMapBenchmark.sh");
       then ();
  end matchcontinue;
end main;

end Main;