
package Util

function applyAndAppend
"@author adrpo
 fun f(x) => y
 fun applyAndAppend(x,f,a) => a @ {(f x)})"
  input Type_a element;
  input FuncTypeType_aToType_b f;
  input list<Type_b> accLst;
  output list<Type_b> outLst;
  replaceable type Type_b subtypeof Any;    
  replaceable type Type_b subtypeof Any;  
  partial function FuncTypeType_aToType_b
    input Type_a inTypeA;
    output Type_b outTypeB;
  end FuncTypeType_aToType_b;
algorithm 
  outLst := matchcontinue(element, f, accLst)
    case(element, f, accLst)
      local Type_b result;
      equation
        result = f(element);
        accLst = listAppend(accLst, {result});
      then accLst;
  end matchcontinue;
end applyAndAppend;


function applyAndCons
"@author adrpo
 fun f(x) => y
 fun applyAndCons(x,f,a) => (f x)::a)"
  input Type_a element;
  input FuncTypeType_aToType_b f;
  input list<Type_b> accLst;
  output list<Type_b> outLst;
  replaceable type Type_b subtypeof Any;    
  replaceable type Type_b subtypeof Any;  
  partial function FuncTypeType_aToType_b
    input Type_a inTypeA;
    output Type_b outTypeB;
  end FuncTypeType_aToType_b;
algorithm 
  outLst := matchcontinue(element, f, accLst)
    case(element, f, accLst)
      local Type_b result;
      equation
        result = f(element);
      then result::accLst;
  end matchcontinue;
end applyAndCons;


function listApplyAndFold 
"@author adrpo
 listApplyAndFold(list<'a>, apply:(x,f,a) => (f x)::a, f:a=>b, accumulator) => list<'b>"
  input list<Type_a> lst;
  input FoldFunc foldFunc;
  input FuncType_a2Type_b typeA2typeB;
  input list<Type_b> accumulator;
  output list<Type_b> result;
  replaceable type Type_a subtypeof Any;
  replaceable type Type_b subtypeof Any;
  partial function FoldFunc
    input Type_a inElement;
    input FuncType_a2Type_b typeA2typeB;
    input list<Type_b> accumulator;
    output list<Type_b> outLst;
    partial function FuncType_a2Type_b
      input Type_a inElement;
      output Type_b outElement;
    end FuncType_a2Type_b;      
  end FoldFunc;
  partial function FuncType_a2Type_b
    input Type_a inElement;
    output Type_b outElement;
  end FuncType_a2Type_b;  
algorithm 
  result :=
  matchcontinue (lst,foldFunc,typeA2typeB,accumulator)
    local
      list<Type_b> foldArg1, foldArg2;
      list<Type_a> rest;
      Type_a hd;
    case ({},_,_,accumulator) then accumulator; 
    case (hd :: rest,foldFunc,typeA2typeB,accumulator)
      equation 
        foldArg1 = foldFunc(hd,typeA2typeB,accumulator);
        foldArg2 = listApplyAndFold(rest, foldFunc, typeA2typeB, foldArg1);        
      then
        foldArg2;
  end matchcontinue;
end listApplyAndFold;


function listMap_impl_0 "function: listMap
  Takes a list and a function over the elements of the lists, which is applied
  for each element, producing a new list.
  Example: listMap({1,2,3}, intString) => { \"1\", \"2\", \"3\"}"
  input list<Type_a> inTypeALst;
  input FuncTypeType_aToType_b inFuncTypeTypeAToTypeB;
  output list<Type_b> outTypeBLst;
  replaceable type Type_a subtypeof Any;
  partial function FuncTypeType_aToType_b
    input Type_a inTypeA;
    output Type_b outTypeB;
    replaceable type Type_b subtypeof Any;
  end FuncTypeType_aToType_b;
  replaceable type Type_b subtypeof Any;
algorithm 
  /* implementation 0 - adrpo: seems to be the fastest */  
  outTypeBLst := listApplyAndFold(inTypeALst, applyAndAppend, inFuncTypeTypeAToTypeB, {});
end listMap_impl_0;

function listMap_impl_1 "function: listMap
  Takes a list and a function over the elements of the lists, which is applied
  for each element, producing a new list.
  Example: listMap({1,2,3}, intString) => { \"1\", \"2\", \"3\"}"
  input list<Type_a> inTypeALst;
  input FuncTypeType_aToType_b inFuncTypeTypeAToTypeB;
  output list<Type_b> outTypeBLst;
  replaceable type Type_a subtypeof Any;
  partial function FuncTypeType_aToType_b
    input Type_a inTypeA;
    output Type_b outTypeB;
    replaceable type Type_b subtypeof Any;
  end FuncTypeType_aToType_b;
  replaceable type Type_b subtypeof Any;
algorithm 
  /* implementation 1 */
  outTypeBLst := listReverse(listApplyAndFold(inTypeALst, applyAndCons, inFuncTypeTypeAToTypeB, {}));
end listMap_impl_1;

function listMap_impl_2
"@author adrpo
 this will work in O(2n) due to listReverse"
  replaceable type TypeVar subtypeof Any;
  input  list<TypeVar> inLst;
  input  list<TypeVar> accumulator;
  input  FuncTypeTypeVarToTypeVar fn;  
  output list<TypeVar> outLst;
  partial function FuncTypeTypeVarToTypeVar
    input TypeVar inTypeA;
    output TypeVar outTypeA;
    replaceable type TypeVar subtypeof Any;
  end FuncTypeTypeVarToTypeVar;
algorithm
  outLst := matchcontinue(inLst, accumulator, fn)
    local
      TypeVar hd, hdChanged;
      list<TypeVar> rest, l, result;
    case ({}, l, _) then listReverse(l);
    case (hd::rest, l, fn)
      equation
        hdChanged = fn(hd);
        l = hdChanged::l;
        result = listMap_impl_2(rest, l, fn);
    then 
        result;
  end matchcontinue;
end listMap_impl_2;

public function listMap_impl_3 "function: listMap
  Takes a list and a function over the elements of the lists, which is applied
  for each element, producing a new list.
  Example: listMap({1,2,3}, intString) => { \"1\", \"2\", \"3\"}"
  input list<Type_a> inTypeALst;
  input FuncTypeType_aToType_b inFuncTypeTypeAToTypeB;
  output list<Type_b> outTypeBLst;
  replaceable type Type_a subtypeof Any;
  partial function FuncTypeType_aToType_b
    input Type_a inTypeA;
    output Type_b outTypeB;
    replaceable type Type_b subtypeof Any;
  end FuncTypeType_aToType_b;
  replaceable type Type_b subtypeof Any;
algorithm 
  /* implementation 2 */
  outTypeBLst := 
  matchcontinue (inTypeALst,inFuncTypeTypeAToTypeB)
    local
      Type_b f_1;
      list<Type_b> r_1;
      Type_a f;
      list<Type_a> r;
      FuncTypeType_aToType_b fn;
    case ({},_) then {}; 
    case ((f :: r),fn)
      equation 
        f_1 = fn(f);
        r_1 = listMap_impl_3(r, fn);
      then
        (f_1::r_1);
  end matchcontinue;
end listMap_impl_3;

function printStringList
  input  list<String> inLst;
algorithm
  _ := matchcontinue(inLst)
    local String hd; list<String> rest;
    case ({}) then ();
    case (hd::rest)
      equation
        print(hd +& " ");
        printStringList(rest);
    then 
        ();
  end matchcontinue;
end printStringList;

function generateStringList
  input Integer size, accumulatorSize;
  input list<String> accumulatorLst;
  output list<String> outLst;
algorithm
  outLst := matchcontinue(size, accumulatorSize, accumulatorLst)
    local 
      Integer n, i; String str; list<String> a;
    case(n, i, a) equation n <= i = true; then a;
    case(n, i, a) equation 
       n > i = true; i = i + 1;
       str = "a"; // realString(clock());
       a = str::a; a = generateStringList(n, i, a);
     then a;     
  end matchcontinue;
end generateStringList;

function identityString
"@author adrpo
 do nothing, just the identity function"
  input String x;
  output String y;
algorithm
  y := x;
end identityString;

function generateIntegerList
  input Integer size, accumulatorSize;
  input list<Integer> accumulatorLst;
  output list<Integer> outLst;
algorithm
  outLst := matchcontinue(size, accumulatorSize, accumulatorLst)
    local 
      Integer n, i, val; list<Integer> a;
    case(n, i, a) equation n <= i = true; then a;
    case(n, i, a) equation 
       n > i = true; i = i + 1;
       val = realInt(clock());
       a = val::a; a = generateIntegerList(n, i, a);
     then a;     
  end matchcontinue;
end generateIntegerList;

function identityInteger
"@author adrpo
 do nothing, just the identity function"
  input Integer x;
  output Integer y;
algorithm
  y := x;
end identityInteger;

function generateRealList
  input Integer size, accumulatorSize;
  input list<Real> accumulatorLst;
  output list<Real> outLst;
algorithm
  outLst := matchcontinue(size, accumulatorSize, accumulatorLst)
    local 
      Integer n, i; Real val; list<Real> a;
    case(n, i, a) equation n <= i = true; then a;
    case(n, i, a) equation 
       n > i = true; i = i + 1;
       val = clock();
       a = val::a; a = generateRealList(n, i, a);
     then a;     
  end matchcontinue;
end generateRealList;

function identityReal
"@author adrpo
 do nothing, just the identity function"
  input Real x;
  output Real y;
algorithm
  y := x;
end identityReal;

end Util;